/***************************************************************************//**
*   @file   cn0216.c
*   @brief  Implementation of CN0216 Driver for NIOS processor.
*   @author DBogdan (dragos.bogdan@analog.com)
*
********************************************************************************
* Copyright 2012(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
* LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
********************************************************************************
*   SVN Revision: $WCREV$
*******************************************************************************/
#define CN0216_GLOBALS

/******************************************************************************/
/* Include Files															  */
/******************************************************************************/
#include "cn0216.h"
#include <altera_avalon_spi_regs.h>		// SPI
#include <altera_avalon_pio_regs.h>		// PIO access macros.
#include <altera_avalon_timer_regs.h>	// Timer
#include <system.h>
#if (USE_CN0216 == 1)

/******************************************************************************/
/*  Private variables                                                         */
/******************************************************************************/
volatile unsigned int 	conversionData[100];						// Acquired data.
volatile unsigned int	*dataLocation = (unsigned int *)0x1BFF0;	// This variable will contain the memory
																	// location of the conversion[0].

/******************************************************************************/
/*  Aliases for the uC Probe data                                             */
/******************************************************************************/
volatile unsigned char*	oneSampleButton			= &CN0216_data[0].uchData;	// Acquire One Samples button.
volatile unsigned int*	oneSampleData			= &CN0216_data[1].uiData;	// Received data from AD7791.
volatile unsigned char* multipleSamplesButton	= &CN0216_data[2].uchData;	// Acquire Multiple Samples button.
volatile unsigned int*	acquisitionTime			= &CN0216_data[3].uiData;	// Acquisition time.

/***************************************************************************//**
 * @brief Reads data from AD7791.
 *
 * @param regAddress - Address of the register.
 * @param bytesNumber - Number of bytes.
 *
 * @return readData - Received data.
*******************************************************************************/
unsigned int AD7791_Read(unsigned char regAddress,
						 unsigned char bytesNumber)
{
	unsigned int receivedData = 0x0;
	unsigned char command = 0x0;
	command = AD7791_COMM_READ +
			  AD7791_COMM_ADDR(regAddress);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE,
									  AD7791_SYNC_PIN);	// AD7791_SYNC_PIN Low.
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);	// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, command);	// Put data in txdata register.
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);	// Wait until the interface has finished transmitting.
	IORD_ALTERA_AVALON_SPI_RXDATA(CN0216_SPI);		// Discard any stale data, in case previous communication was interrupted.
	while(bytesNumber--)
	{
		receivedData = receivedData << 8;
		while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
			   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);	// Wait until txdata register is empty.
		IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, 0x00);	// Generate 8 clocks.
		while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
			   ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);			// Wait until rxdata register is full.
		receivedData += IORD_ALTERA_AVALON_SPI_RXDATA(CN0216_SPI);	// Read data from rxdata register.
	}
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);			// Wait until the interface has finished transmitting.
	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE,
									AD7791_SYNC_PIN);		// AD7791_SYNC_PIN High.

	return(receivedData);
}

/***************************************************************************//**
 * @brief Writes data to AD7791.
 *
 * @param regAddress - Address of the register.
 * @param bytesNumber - Number of bytes.
 * @param regData - Data that will be loaded.
 *
 * @return None.
*******************************************************************************/
void AD7791_Write(unsigned char regAddress,
				  unsigned char bytesNumber,
				  unsigned int regData)
{
	unsigned char command = 0x0;
	unsigned int mask = 0x0;
	unsigned char currentByte = 0x0;
	command = AD7791_COMM_WRITE +
			  AD7791_COMM_ADDR(regAddress);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE,
									  AD7791_SYNC_PIN);			// AD7791_SYNC_PIN Low.
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, command);			// Put data in txdata register.
	while(bytesNumber--)
	{
		mask = 0xFF << (bytesNumber * 8);
		currentByte = (regData & mask) >> (bytesNumber * 8);
		while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
			   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);		// Wait until txdata register is empty.
		IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, currentByte);	// Put data in txdata register.
	}
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);				// Wait until the interface has finished transmitting.
	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE,
									AD7791_SYNC_PIN);			// AD7791_SYNC_PIN High.
}

/***************************************************************************//**
 * @brief Acquires "samplesNumber" samples and store them in conversionData[].
 *
 * @param samplesNumber - Number of samples.
 *
 * @return elapsedTime - Acquisition time.
*******************************************************************************/
unsigned int AD7791_Convertion(unsigned int samplesNumber)
{
	unsigned char bytesNumber = 0x0;
	unsigned int sample = 0x0;
	unsigned int receivedData = 0x0;
	unsigned int elapsedTime = 0x0;
	IOWR_ALTERA_AVALON_TIMER_PERIODL(TIMER_BASE, 0xFFFF);	// Count down from 0xFFFFFFFF.
	IOWR_ALTERA_AVALON_TIMER_PERIODH(TIMER_BASE, 0xFFFF);
    IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_BASE,
    								 ALTERA_AVALON_TIMER_CONTROL_START_MSK |	// Start timer.
    								 ALTERA_AVALON_TIMER_CONTROL_START_MSK);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(SYNC_SIGNALS_BASE,
									  AD7791_SYNC_PIN);		// AD7791_SYNC_PIN Low.
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);		// Wait until txdata register is empty.
	IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, 0x3C);		// Put data in txdata register.
	while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
		   ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);			// Wait until the interface has finished transmitting.
	for(sample = 0;sample < samplesNumber;sample ++)
	{
		while(IORD_ALTERA_AVALON_PIO_DATA(ADC_DATA_READY_BASE) &
			  AD7791_RDY_PIN);
		bytesNumber = 3;
		receivedData = 0;
		while(bytesNumber--)
		{
			receivedData = receivedData << 8;
			while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
				   ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);			// Wait until txdata register is empty.
			IOWR_ALTERA_AVALON_SPI_TXDATA(CN0216_SPI, 0x00);			// Generate 8 clocks.
			while((IORD_ALTERA_AVALON_SPI_STATUS(CN0216_SPI) &
				   ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);			// Wait until rxdata register is full.
			receivedData += IORD_ALTERA_AVALON_SPI_RXDATA(CN0216_SPI);	// Read data from rxdata register.
		}
		conversionData[sample] = receivedData;
	}
	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE,
									AD7791_SYNC_PIN);// AD7791_SYNC_PIN High.
	IOWR_ALTERA_AVALON_TIMER_SNAPL(TIMER_BASE, 0x0); // Latch the current timer value to the SNAPL and SNAPH registers.
	elapsedTime = (IORD_ALTERA_AVALON_TIMER_SNAPH(TIMER_BASE) << 16) +
				   IORD_ALTERA_AVALON_TIMER_SNAPL(TIMER_BASE);
	elapsedTime = 0xFFFFFFFF - elapsedTime;
    IOWR_ALTERA_AVALON_TIMER_CONTROL(TIMER_BASE,
    								 ALTERA_AVALON_TIMER_CONTROL_STOP_MSK);	// Stop timer.
    elapsedTime = elapsedTime * 0.01; // us

	return(elapsedTime);
}

/***************************************************************************//**
 * @brief Initializes SPI and initial values for CN0216 board.
 *
 * @param None.
 *
 * @return Always returns success.
*******************************************************************************/
int CN0216_Init(void)
{
	/* Deselect AD7171. */
	IOWR_ALTERA_AVALON_PIO_SET_BITS(SYNC_SIGNALS_BASE, AD7791_SYNC_PIN);
	SetCtrl(CN0216_SPI_CFG);	// Setup SPI interface.
	*dataLocation = (unsigned int) &conversionData[0];

	return(0);
}

/**************************************************************************//**
 * @brief Updates the device's monitored and controlled parameters.
 *
 * @param None.
 *
 * @return Always returns success.
******************************************************************************/
int CN0216_UpdateParameters(void)
{
	if(*oneSampleButton)
	{
		AD7791_Convertion(1);
		*oneSampleData = conversionData[0];
		*oneSampleButton = 0;
	}
	if(*multipleSamplesButton)
	{
		*acquisitionTime = AD7791_Convertion(100);
		*multipleSamplesButton = 0;
	}

	return (0);
}
#endif /* USE_CN0216 */
